#include "Stepper.h"

/*----- Variables and Pins -----*/
#define STEPS_PER_REV 2048  // Full steps per revolution for the 28BYJ-48 motor
#define FRACTIONAL_DEFAULT 128 // Default fractional step count (can be changed during runtime)

int steps_per_fraction;    // Steps per fractional rotation
bool is_running = false;   // Motor state
bool is_clockwise = true;  // Motor direction
unsigned long last_step_time = 0; // Tracks the last time the motor stepped
const int pause_duration = 2000;  // Pause duration in milliseconds

/*-----( Declare Stepper Object )-----*/
// Initialize the stepper motor object with proper pin sequence
Stepper small_stepper(STEPS_PER_REV / 64, 8, 10, 9, 11);

void setup() {
  Serial.begin(9600);  // Initialize Serial Communication
  steps_per_fraction = STEPS_PER_REV / FRACTIONAL_DEFAULT; // Set initial fractional step size
  small_stepper.setSpeed(500); // Set motor speed (max 500 rpm)
  Serial.println("Stepper Motor Control Ready");
  Serial.println("Commands: RUN, STOP, FLIP, SETFRACTION=<number>");
}

void loop() {
  // Check for serial input
  if (Serial.available() > 0) {
    String command = Serial.readStringUntil('\n');
    command.trim(); // Remove any extra spaces or newline characters

    if (command.equalsIgnoreCase("RUN")) {
      is_running = true;
      Serial.println("Motor running...");
    } 
    else if (command.equalsIgnoreCase("STOP")) {
      is_running = false;
      Serial.println("Motor stopped.");
    } 
    else if (command.equalsIgnoreCase("FLIP")) {
      is_clockwise = !is_clockwise;
      Serial.println("Motor direction flipped.");
    } 
    else if (command.startsWith("SETFRACTION=")) {
      int fraction = command.substring(12).toInt();
      if (fraction > 0 && fraction <= STEPS_PER_REV) {
        steps_per_fraction = STEPS_PER_REV / fraction;
        Serial.print("Fractional step set to: ");
        Serial.println(fraction);
      } else {
        Serial.println("Invalid fraction value. It must be between 1 and 2048.");
      }
    } 
    else {
      Serial.println("Unknown command.");
    }
  }

  // Handle motor operation
  if (is_running && millis() - last_step_time >= pause_duration) {
    int steps_to_take = is_clockwise ? steps_per_fraction : -steps_per_fraction;
    small_stepper.step(steps_to_take);
    last_step_time = millis();
    Serial.println("Motor stepped.");
  }
}
